<?php
/**
 * Implementation of the Data Source class
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@seznam.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/bsd-license.php new BSD license
 */

/**
 * class for the data source of our application, is implemented as sigleton.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@seznam.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/bsd-license.php new BSD license
 */
class DataSource
{
 /**
  * Singleton instance
  * @var DataSource
  */
 private static $_instance = null;

 /**
 * Database connection
 * @var Zend_Db_Adapter_Abstract
 */
 private $_db;
 
//==============================================================================
/**
* Init and prepare database adapter.
* Singleton pattern implementation makes "new" unavailable.
* @param array $dbConOptions array with database connections parameters
*/
private function __construct($dbConOptions=null)
{
 if (is_null($dbConOptions)) {
  $dbConOptions=self::_getDbOptionsFromCfg();
 }
 $this->_db = Zend_Db::factory('Pdo_Mysql',$dbConOptions);
 //set correct charset
 self::setNames($this->_db);
}

//==============================================================================
/**
 * Singleton pattern implementation makes "clone" unavailable
 * @return void
 */
private function __clone()
{}

//==============================================================================
/**
 * Singleton pattern implementation.
 * Only one instance for a whole system
 * @return DataSource
 */
public static function getInstance()
{
 if (null === self::$_instance) {
  self::$_instance = new self();
 }
 return self::$_instance;
}

//==============================================================================
/**
 * Gets options for the database connection from the application configuration.
 * @return array asociative array with keys: host, username, password, dbname
 */
private static function _getDbOptionsFromCfg()
{
 $config = Zend_Registry::get("conf");

 return array (
  "host"=>$config->database->host,
  "username"=>$config->database->username,
  "password"=>$config->database->password,
  "dbname"=>$config->database->dbName
  );
}

//==============================================================================
/**
 * Set correct mysql charset
 * @param $dbConn database connection
 */
public static function setNames(Zend_Db_Adapter_Abstract $dbConn)
{
 //get mysql version
 $ver=$dbConn->fetchOne("select version()");
 if (!preg_match("/^4\.0/",$ver)) {
  //set names to latin2
  //echo "set names to latin2<br>";
  $dbConn->query("SET NAMES 'latin2'");
 }
}

//==============================================================================
/**
 * Gets database adapter for current data source.
 * @return Zend_Db_Adapter_Abstract
 */
public function getAdapter()
{
 return $this->_db;
}

//==============================================================================
/**
 * Add SQL limit command for the pagination.
 * @param string|Zend_Db_Select $sql command, which will be extended
 * @param int $pageNo current page number, begins with no.1
 * @param int $perPageItems row per page
 * @return string|Zend_Db_Select
 */
public function addSQLlimit($sql,$pageNo,$perPageItems)
{
 if (is_numeric($pageNo) && is_numeric($perPageItems)) {
  if ($pageNo<1) throw new RangeException("page number must be be greater then 0");
  if ($sql instanceof Zend_Db_Select) {
   return $sql->limitPage($pageNo,$perPageItems);
  }

  //string
  $pageNo--;
  return $this->_db->limit($sql,$perPageItems,$pageNo*$perPageItems);
 }
}

#END CLASS
}
