<?php
/**
 * Implementation for the helper for month calendar archive
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * The helper for one month calendar.
 *
 * This helper renders inner part of the html table without <table></table>
 * tags.
 *
 * @package koubel homepage
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2008 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class HomeWebApp_View_Helper_MonthCalendarArchive
{
 /**
  * @var Calendar calendar object from the library
  */
 private $_calendar;

 /**
  * @var Zend_Locale
  */
 private $_locale;

 /**
  * localized day names
  * @var array 
  */
 private $_dayNames;

 /**
  * view from which this helper is called
  * @var array
  */
 private $_view;
 
//==============================================================================
/**
 * Initilizes the calendar, locale and names for a days
 */
public function __construct()
{
 $this->_calendar=new Calendar();
 $this->_locale=HomeWebApp::getInstance()->getLocale();
 $this->_dayNames=$this->_locale->getTranslationList("Day",null,
  array('gregorian', 'format', 'abbreviated'));

 //adjust daynames
 foreach($this->_dayNames as $key=>$val) {
  $val=iconv("utf-8","iso-8859-2",$val);
  if (strlen($val)>2) $val=substr($val,0,2);
  $this->_dayNames[$key]=$val;
 }

 switch ($this->_locale->toString()) {
  case "cs_CZ":
   $this->_calendar->setStartDay(1);
  default:
   break;
 }
}

//==============================================================================
/**
 * Sets view, is called from when the helper is instantiated.
 */
public function setView(Zend_View_Interface $view)
{
 $this->_view = $view;
}

//==============================================================================
/**
 * Main helper function
 * @return string xhtml for the one month table without <table></table>
 * tags
 */
public function monthCalendarArchive(ArticleMonthArchive $archive)
{
 $this->_calendar->setMonth( $archive->getMonth() );
 $this->_calendar->setYear( $archive->getYear() );

 Calendar::shiftDayNames($this->_calendar->getStartDay(),$this->_dayNames);

 //renders head
 $this->_view->archiveItem=$archive;
 $xHtml=$this->_view->render("calendarArchiveHead.phtml");
 unset($this->_view->archiveItem);

 $xHtml.="<tr class=\"dayhead\">\n";
 foreach($this->_dayNames as $dayName) {
  $xHtml.="<td>$dayName</td>";
 }
 $xHtml.="</tr>\n";

 $daysInMonth=$this->_calendar->getMonthArray();

 if ($this->_calendar->getMonth()==date("n")
   && $this->_calendar->getYear()==date("Y")
 ) $isCurrentMonth=true;
 else $isCurrentMonth=false;

 $currentDay=date("j");

 foreach($daysInMonth as $week) {
  $xHtml.="<tr>\n";
  foreach($week as $day) {
   $xHtml.= "<td>";

   if (is_null($day)) {
    $xHtml.="&nbsp;";
   } else {
    $dayLink=$this->_getDayLink($day,$archive->getDayArchive($day));
    if ($isCurrentMonth && $day==$currentDay) {
     $xHtml.="<div style=\"margin:1px; padding:1px; border:1px solid black\">"
      . $dayLink
      . "</div>";
    } else {
     $xHtml.= $dayLink;
    }
   }

   $xHtml.="</td>\n";
  }
  $xHtml.="</tr>\n";
 }

 //add calendar navigator
 $xHtml.=$this->_getNavigator($archive);

 return $xHtml;
}

//==============================================================================
/**
 * Returns links for a one day
 * @param int $day (1-31)
 * @param ArticleDayArchive $dayArchive articles archive for $day
 * @return string <a href> link for $day
 */
private function _getDayLink($day,ArticleDayArchive $dayArchive=null)
{
 if (is_null($dayArchive)) return $day;

 $urlEnum=AppUrlEnum::getInstance();
 $urlEnum->setBlogArchiveYear($this->_calendar->getYear());
 $urlEnum->setBlogArchiveMonth($this->_calendar->getMonth());
 $urlEnum->setBlogArchiveDay($day);

 $title="";
 $articles=$dayArchive->getArticles();
 $printArticleOrder=(count($articles)>1) ? true : false;
 for($i=0;$i<count($articles);$i++) {
  if ($printArticleOrder) {
   if ($i>0) $title.="  ";
   $title.=($i+1).": ";
  }
  $title.=$articles[$i]->getTitle();
 }
 $title=htmlspecialchars($title,ENT_QUOTES);
 
 $link="<a href=\"".$urlEnum->blogArchive."\" title=\"$title\">$day</a>";
 return $link;
}

//==============================================================================
/** Returns the navigator markup for the next and previous months.
 * Only month with some articles are considered.
 *
 * @param ArticleMonthArchive $archive article archive for current viewed month.
 * @return string xhmth markup - see calendarNavRow.phtml template
 */
private function _getNavigator(ArticleMonthArchive $archive)
{
 $urlEnum=AppUrlEnum::getInstance();

 //no days in the navigator archive link
 $urlEnum->setBlogArchiveDay(null);

 //previous month - only month with at least one article are returned
 //months without articles are skipped
 $prevArchive=$archive->getPrevMonthStat();
 if (!is_null($prevArchive)) {
  $this->_calendar->setMonth($prevArchive->getMonth());
  $this->_calendar->setYear($prevArchive->getYear());
  $this->_view->prevMonth=$prevArchive->getMonth();
  $this->_view->prevYear=$prevArchive->getYear();
  $this->_view->prevMonthName=$this->_getLongMonthName($this->_calendar->getMonth());
  if ($this->_calendar->getYear()!=$archive->getYear())
   $this->_view->prevMonthName.= " ".substr($this->_calendar->getYear(),-2);
 }

 //next month - only month with at least one article are returned
 //months without articles are skipped
 $nextArchive=$archive->getNextMonthStat();
 if (!is_null($nextArchive)) {
  $this->_calendar->setMonth($nextArchive->getMonth());
  $this->_calendar->setYear($nextArchive->getYear());
  $this->_view->nextMonth=$nextArchive->getMonth();
  $this->_view->nextYear=$nextArchive->getYear();
  $this->_view->nextMonthName=$this->_getLongMonthName($this->_calendar->getMonth());
  if ($this->_calendar->getYear()!=$archive->getYear())
   $this->_view->nextMonthName.= " ".substr($this->_calendar->getYear(),-2);
 }

 return $this->_view->render("calendarNavRow.phtml");
}

//==============================================================================
/**
 * Gets localized month name for the particular month.
 * @param int $month (1-12)
 */
private function _getLongMonthName($month)
{
 return iconv("utf-8","iso-8859-2",Utils::getMonthFirstCaseName($month,$this->_locale));
}

#END CLASS
}
